==> frogml-1.1.0-py3-none-any.whl: frogml/sdk/model_version/utils/storage.py <==
import logging
import os
from datetime import datetime, timezone
from platform import python_version
from typing import Dict, Optional, List, Callable, cast, Any

from frogml.core.exceptions import FrogmlException
from frogml.core.utils.proto_utils import ProtoUtils
from frogml._proto.jfml.model_version.v1.model_version_framework_pb2 import (
    ModelVersionFramework,
)
from frogml.storage.frog_ml import SerializationMetadata, FrogMLStorage
from frogml.storage.models.model_manifest import ModelManifest
from frogml.storage.models.frogml_model_version import FrogMLModelVersion

from frogml.sdk.model_version.constants import FROGML_LOG_LEVEL_ENVAR_NAME
from frogml.sdk.model_version.finders.model_class_info import ModelClassInfo
from frogml.sdk.model_version.finders.model_finder import ModelFinder
from frogml.sdk.model_version.modifiers.model_modifier import ModelModifier
from frogml.sdk.model_version.utils.dependencies_tools import _dependency_files_handler
from frogml.sdk.model_version.utils.files_tools import (
    _zip_dir,
    _copy_dir_without_ignored_files,
    _remove_dir,
)
from frogml.sdk.model_version.utils.jml.customer_client import JmlCustomerClient
from frogml.sdk.model_version.utils.validations import _validate_load_model

_STORAGE_MODEL_ENTITY_TYPE = "model"
_PYTHON_RUNTIME = "python"

_logger = logging.getLogger(__name__)
_logger.setLevel(os.getenv(FROGML_LOG_LEVEL_ENVAR_NAME) or logging.INFO)


def _get_model_metadata(
    model_flavor: str,
    model_flavor_version: str,
    serialization_format: str,
) -> SerializationMetadata:
    return SerializationMetadata(
        framework=model_flavor,
        framework_version=model_flavor_version,
        serialization_format=serialization_format,
        runtime=_PYTHON_RUNTIME,
        runtime_version=python_version(),
    )


def _get_model_info_from_artifactory(
    repository: str,
    model_name: str,
    version: str,
) -> Dict:
    return FrogMLStorage().get_entity_manifest(
        entity_type=_STORAGE_MODEL_ENTITY_TYPE,
        repository=repository,
        entity_name=model_name,
        version=version,
        namespace=None,
    )


def _download_model_version_from_artifactory(
    model_flavor: str,
    repository: str,
    model_name: str,
    version: str,
    model_framework: str,
    download_target_path: str,
    deserializer: Callable,
):
    """
    Download model version from artifactory
    :param model_flavor: model flavor files/catboost etc.
    :param repository: repository name
    :param model_name: the name of the model
    :param version: version of the model
    :param download_target_path: the path to download the model to
    :param model_framework: model framework files/catboost etc.
    :return: Loaded model
    """
    _validate_load_model(
        repository=repository,
        model_name=model_name,
        version=version,
        model_framework=model_framework,
        model_flavor=model_flavor,
    )

    FrogMLStorage().download_model_version(
        repository=repository,
        model_name=model_name,
        version=version,
        target_path=download_target_path,
    )
    return deserializer()


def _log_model(
    model_name: str,
    target_dir: str,
    model_flavor: str,
    framework_version: str,
    full_model_path: str,
    serialization_format: str,
    repository: str,
    version: Optional[str] = None,
    properties: Optional[Dict[str, str]] = None,
    dependencies: Optional[List[str]] = None,
    code_dir: Optional[str] = None,
    parameters: Optional[Dict[str, Any]] = None,
    metrics: Optional[Dict[str, Any]] = None,
) -> None:
    jml_customer_client = JmlCustomerClient()

    if not version:
        version = datetime.now(timezone.utc).strftime("%Y-%m-%d-%H-%M-%S-%f")[:-3]

    project_key: str = jml_customer_client.get_project_key(repository)
    model_version_framework: ModelVersionFramework = (
        ProtoUtils.model_framework_from_file_format(
            serialization_format, framework_version
        )
    )

    is_customer_exists_in_jml: bool = (
        jml_customer_client.check_if_customer_exists_in_jml()
    )
    if is_customer_exists_in_jml:
        jml_customer_client.validate_model_version(
            project_key, repository, model_name, version, model_version_framework
        )

    properties = merge_properties_parameters_metrics(metrics, parameters, properties)
    dependencies = _dependency_files_handler(
        dependencies=dependencies, target_dir=target_dir
    )
    metadata = _get_model_metadata(
        model_flavor, framework_version, serialization_format
    )
    zipped_code_path = build_zip_code_path(
        code_dir, target_dir, model_flavor, model_name, repository, version
    )

    frog_ml_model_version: FrogMLModelVersion = __log_model_to_artifactory(
        dependencies,
        full_model_path,
        model_name,
        properties,
        repository,
        version,
        metadata,
        zipped_code_path,
    )

    if is_customer_exists_in_jml:
        jml_customer_client.register_model_version_in_jml(
            project_key,
            repository,
            model_name,
            version,
            model_version_framework,
            cast(ModelManifest, frog_ml_model_version.entity_manifest),
            parameters,
            metrics,
        )


def build_zip_code_path(
    code_dir_path: Optional[str],
    parent_dir_path: str,
    model_flavor: str,
    model_name: str,
    repository: str,
    version: str,
) -> Optional[str]:
    """
    Build the path to the zipped code directory.
    Before zipping, the code directory is copied to a target directory and filtered to remove ignored files.
    Then, the code directory is processed to find and modify model classes.
    The resulting code directory is zipped and the path to the zipped code directory is returned.

    :param code_dir_path: The path to the code directory
    :param model_flavor: The model flavor
    :param model_name: The model name
    :param repository: The repository key
    :param parent_dir_path: The parent directory of the zip file
    :param version: The model version
    :return: The path to the zipped code directory
    """
    zipped_code_path: Optional[str] = None

    if code_dir_path is None:
        return zipped_code_path

    filtered_copied_dir_path: str = _copy_dir_without_ignored_files(
        code_dir_path, parent_dir_path
    )
    try:
        # Jira Ticket: MLB-256 https://jfrog-int.atlassian.net/browse/MLB-256
        # process_code_directory(
        #     filtered_copied_dir_path, repository, model_name, version, model_flavor
        # )
        zipped_code_path = _zip_dir(
            root_dir=parent_dir_path, dir_to_zip=filtered_copied_dir_path
        )
    except Exception as e:
        raise FrogmlException(f"Failed to zip code directory: {e}") from e
    finally:
        _remove_dir(filtered_copied_dir_path)

    return zipped_code_path


def merge_properties_parameters_metrics(
    metrics: Optional[Dict[str, Any]] = None,
    parameters: Optional[Dict[str, Any]] = None,
    properties: Optional[Dict[str, str]] = None,
) -> Dict[str, str]:
    result = {}
    if properties is not None:
        result.update(properties)

    if parameters is not None:
        result.update(parameters)

    if metrics is not None:
        result.update(metrics)

    result = {key: str(value) for key, value in result.items()}

    return result


def __log_model_to_artifactory(
    dependencies: Optional[List[str]],
    full_model_path: str,
    model_name: str,
    properties: Optional[Dict[str, str]],
    repository: str,
    version: str,
    metadata: SerializationMetadata,
    code_archive_file_path: Optional[str],
) -> FrogMLModelVersion:
    return FrogMLStorage().upload_model_version(
        repository=repository,
        model_name=model_name,
        model_path=full_model_path,
        model_type=metadata,
        version=version,
        properties=properties,
        dependencies_files_paths=dependencies,
        code_archive_file_path=code_archive_file_path,
    )


def process_code_directory(
    code_dir: str, repository: str, model_name: str, version: str, framework_name: str
):
    """
    Process a directory of Python files to find and modify model classes.

    :param code_dir: Directory containing Python files to analyze
    :param repository: Repository key to use in the template
    :param model_name: Model name to use in the template
    :param version: Model version to use in the template
    :param framework_name: The name of the framework used to train the model

    :raises ValueError: If no valid model class is found or if the model class doesn't have a model field
    """
    model_class_info: ModelClassInfo = ModelFinder.find_model_class(code_dir)

    if model_class_info is None:
        raise ValueError(
            "No class inheriting from FrogMlModel found in the code directory"
        )

    if model_class_info.model_field_name is None:
        raise ValueError(
            f"The class {model_class_info.class_name} does not have a model field (model, _model, or __model)"
        )

    ModelModifier.modify_model_file(
        model_class_info,
        repository,
        model_name,
        version,
        framework_name,
    )

    print(
        f"Successfully modified {model_class_info.file_path} for class {model_class_info.class_name}"
    )

==> frogml-1.1.3-py3-none-any.whl: frogml/sdk/model_version/utils/storage.py <==
import logging
import os
from datetime import datetime, timezone
from platform import python_version
from typing import Dict, Optional, List, Callable, cast, Any

from frogml.core.exceptions import FrogmlException
from frogml.core.utils.proto_utils import ProtoUtils
from frogml._proto.jfml.model_version.v1.model_version_framework_pb2 import (
    ModelVersionFramework,
)
from frogml.sdk.model_version.utils.model_log_config import ModelLogConfig
from frogml.storage.frog_ml import SerializationMetadata, FrogMLStorage
from frogml.storage.models.model_manifest import ModelManifest
from frogml.storage.models.frogml_model_version import FrogMLModelVersion

from frogml.sdk.model_version.constants import FROGML_LOG_LEVEL_ENVAR_NAME
from frogml.sdk.model_version.finders.model_class_info import ModelClassInfo
from frogml.sdk.model_version.finders.model_finder import ModelFinder
from frogml.sdk.model_version.modifiers.model_modifier import ModelModifier
from frogml.sdk.model_version.utils.dependencies_tools import _dependency_files_handler
from frogml.sdk.model_version.utils.files_tools import (
    _zip_dir,
    _copy_dir_without_ignored_files,
    _remove_dir,
)
from frogml.sdk.model_version.utils.jml.customer_client import JmlCustomerClient
from frogml.sdk.model_version.utils.validations import (
    _validate_load_model,
    _validate_code_dependencies,
)

_STORAGE_MODEL_ENTITY_TYPE = "model"
_PYTHON_RUNTIME = "python"

_logger = logging.getLogger(__name__)
_logger.setLevel(os.getenv(FROGML_LOG_LEVEL_ENVAR_NAME) or logging.INFO)


def _get_model_metadata(
    model_flavor: str,
    model_flavor_version: str,
    serialization_format: str,
) -> SerializationMetadata:
    return SerializationMetadata(
        framework=model_flavor,
        framework_version=model_flavor_version,
        serialization_format=serialization_format,
        runtime=_PYTHON_RUNTIME,
        runtime_version=python_version(),
    )


def _get_model_info_from_artifactory(
    repository: str,
    model_name: str,
    version: str,
) -> Dict:
    return FrogMLStorage().get_entity_manifest(
        entity_type=_STORAGE_MODEL_ENTITY_TYPE,
        repository=repository,
        entity_name=model_name,
        version=version,
        namespace=None,
    )


def _download_model_version_from_artifactory(
    model_flavor: str,
    repository: str,
    model_name: str,
    version: str,
    model_framework: str,
    download_target_path: str,
    deserializer: Callable,
):
    """
    Download model version from artifactory
    :param model_flavor: model flavor files/catboost etc.
    :param repository: repository name
    :param model_name: the name of the model
    :param version: version of the model
    :param download_target_path: the path to download the model to
    :param model_framework: model framework files/catboost etc.
    :return: Loaded model
    """
    _validate_load_model(
        repository=repository,
        model_name=model_name,
        version=version,
        model_framework=model_framework,
        model_flavor=model_flavor,
    )

    FrogMLStorage().download_model_version(
        repository=repository,
        model_name=model_name,
        version=version,
        target_path=download_target_path,
    )
    return deserializer()


def _log_model(config: ModelLogConfig) -> None:

    _validate_code_dependencies(
        code_dir=config.code_dir,
        dependencies=config.dependencies,
        predict_file=config.predict_file,
    )

    jml_customer_client = JmlCustomerClient()

    if not config.version:
        config.version = datetime.now(timezone.utc).strftime("%Y-%m-%d-%H-%M-%S-%f")[
            :-3
        ]

    project_key: str = jml_customer_client.get_project_key(config.repository)
    model_version_framework: ModelVersionFramework = (
        ProtoUtils.model_framework_from_file_format(
            config.serialization_format, config.framework_version
        )
    )

    is_customer_exists_in_jml: bool = (
        jml_customer_client.check_if_customer_exists_in_jml()
    )
    if is_customer_exists_in_jml:
        jml_customer_client.validate_model_version(
            project_key,
            config.repository,
            config.model_name,
            config.version,
            model_version_framework,
        )

    properties = merge_properties_parameters_metrics(
        config.metrics, config.parameters, config.properties
    )
    dependencies = _dependency_files_handler(
        dependencies=config.dependencies, target_dir=config.target_dir
    )
    metadata = _get_model_metadata(
        config.model_flavor, config.framework_version, config.serialization_format
    )
    zipped_code_path = build_zip_code_path(config.code_dir, config.target_dir)

    frog_ml_model_version: FrogMLModelVersion = __log_model_to_artifactory(
        dependencies,
        config.full_model_path,
        config.model_name,
        properties,
        config.repository,
        config.version,
        metadata,
        zipped_code_path,
    )

    if is_customer_exists_in_jml:
        jml_customer_client.register_model_version_in_jml(
            project_key,
            config.repository,
            config.model_name,
            config.version,
            model_version_framework,
            cast(ModelManifest, frog_ml_model_version.entity_manifest),
            config.parameters,
            config.metrics,
        )


def build_zip_code_path(
    code_dir_path: Optional[str],
    parent_dir_path: str,
) -> Optional[str]:
    """
    Build the path to the zipped code directory.
    Before zipping, the code directory is copied to a target directory and filtered to remove ignored files.
    Then, the code directory is processed to find and modify model classes.
    The resulting code directory is zipped and the path to the zipped code directory is returned.

    :param code_dir_path: The path to the code directory
    :param parent_dir_path: The parent directory of the zip file
    :return: The path to the zipped code directory
    """
    zipped_code_path: Optional[str] = None

    if code_dir_path is None:
        return zipped_code_path

    filtered_copied_dir_path: str = _copy_dir_without_ignored_files(
        code_dir_path, parent_dir_path
    )
    try:
        # Jira Ticket: MLB-256 https://jfrog-int.atlassian.net/browse/MLB-256
        # process_code_directory(
        #     filtered_copied_dir_path, repository, model_name, version, model_flavor
        # )
        zipped_code_path = _zip_dir(
            root_dir=parent_dir_path, dir_to_zip=filtered_copied_dir_path
        )
    except Exception as e:
        raise FrogmlException(f"Failed to zip code directory: {e}") from e
    finally:
        _remove_dir(filtered_copied_dir_path)

    return zipped_code_path


def merge_properties_parameters_metrics(
    metrics: Optional[Dict[str, Any]] = None,
    parameters: Optional[Dict[str, Any]] = None,
    properties: Optional[Dict[str, str]] = None,
) -> Dict[str, str]:
    result = {}
    if properties is not None:
        result.update(properties)

    if parameters is not None:
        result.update(parameters)

    if metrics is not None:
        result.update(metrics)

    result = {key: str(value) for key, value in result.items()}

    return result


def __log_model_to_artifactory(
    dependencies: Optional[List[str]],
    full_model_path: str,
    model_name: str,
    properties: Optional[Dict[str, str]],
    repository: str,
    version: str,
    metadata: SerializationMetadata,
    code_archive_file_path: Optional[str],
) -> FrogMLModelVersion:
    return FrogMLStorage().upload_model_version(
        repository=repository,
        model_name=model_name,
        model_path=full_model_path,
        model_type=metadata,
        version=version,
        properties=properties,
        dependencies_files_paths=dependencies,
        code_archive_file_path=code_archive_file_path,
    )


def process_code_directory(
    code_dir: str, repository: str, model_name: str, version: str, framework_name: str
):
    """
    Process a directory of Python files to find and modify model classes.

    :param code_dir: Directory containing Python files to analyze
    :param repository: Repository key to use in the template
    :param model_name: Model name to use in the template
    :param version: Model version to use in the template
    :param framework_name: The name of the framework used to train the model

    :raises ValueError: If no valid model class is found or if the model class doesn't have a model field
    """
    model_class_info: ModelClassInfo = ModelFinder.find_model_class(code_dir)

    if model_class_info is None:
        raise ValueError(
            "No class inheriting from FrogMlModel found in the code directory"
        )

    if model_class_info.model_field_name is None:
        raise ValueError(
            f"The class {model_class_info.class_name} does not have a model field (model, _model, or __model)"
        )

    ModelModifier.modify_model_file(
        model_class_info,
        repository,
        model_name,
        version,
        framework_name,
    )

    print(
        f"Successfully modified {model_class_info.file_path} for class {model_class_info.class_name}"
    )
